/* User memory dumper by m0skit0 */
/* Dumps user memory */

#include "sdk.h"
#include <psputility.h>

#define USER_START_ADDRESS 0x08800000
#define USER_END_ADDRESS 0x09FFFFFF

#define KERNEL_START_ADDRESS 0x08000000
#define KERNEL_END_ADDRESS 0x083FFFFF


// From HBL lib.c
// Sets a memory region to a specific value
void * memset(void *ptr, int c, size_t size)
{
	byte *p1 = ptr;
	byte *p2 = ptr + size;
	while(p1 != p2) {
		*p1 = c;
		p1++;
	}
    return ptr;
}



// Initializes the savedata dialog loop, opens p5
void p5_open_savedata(int mode)
{
	SceUtilitySavedataParam dialog;

	memset(&dialog, 0, sizeof(SceUtilitySavedataParam));
	dialog.base.size = sizeof(SceUtilitySavedataParam);

    sceUtilityGetSystemParamInt(PSP_SYSTEMPARAM_ID_INT_LANGUAGE, &dialog.base.language); // Prompt language
    sceUtilityGetSystemParamInt(PSP_SYSTEMPARAM_ID_INT_UNKNOWN, &dialog.base.buttonSwap); // X/O button swap

	dialog.base.graphicsThread = 0x11;
	dialog.base.accessThread = 0x13;
	dialog.base.fontThread = 0x12;
	dialog.base.soundThread = 0x10;

	dialog.mode = mode;
	dialog.overwrite = 1;
	dialog.focus = PSP_UTILITY_SAVEDATA_FOCUS_LATEST; // Set initial focus to the newest file (for loading)

	strcpy(dialog.gameName, "DEMO11111");	// First part of the save name, game identifier name
	strcpy(dialog.saveName, "0000");	// Second part of the save name, save identifier name

	char nameMultiple[][20] =	// End list with ""
	{
	 "0000",
	 "0001",
	 "0002",
	 "0003",
	 "0004",
	 ""
	};

	// List of multiple names
	dialog.saveNameList = nameMultiple;

	strcpy(dialog.fileName, "DATA.BIN");	// name of the data file

	// Allocate buffers used to store various parts of the save data
	dialog.dataBuf = NULL;//malloc_p2(0x20);
	dialog.dataBufSize = 0;//0x20;
	dialog.dataSize = 0;//0x20;

	sceUtilitySavedataInitStart(&dialog);

	// Wait for the dialog to initialize
	while (sceUtilitySavedataGetStatus() < 2)
		sceDisplayWaitVblankStart();
}



// Runs the savedata dialog loop
void p5_close_savedata()
{
	int running = 1;
	int last_status = -1;

	while(running) 
	{
		int status = sceUtilitySavedataGetStatus();

		switch(status) 
		{
			case PSP_UTILITY_DIALOG_VISIBLE:
				sceUtilitySavedataUpdate(1);
				break;

			case PSP_UTILITY_DIALOG_QUIT:
				sceUtilitySavedataShutdownStart();
				break;

			case PSP_UTILITY_DIALOG_NONE:
				running = 0;
				break;

			case PSP_UTILITY_DIALOG_FINISHED:
				break;
		}

		sceDisplayWaitVblankStart();
	}
}


// Entry point
void _start(unsigned long, unsigned long *) __attribute__ ((section (".text.start")));
void _start(unsigned long arglen, unsigned long *argp)
{
	SceUID dump_fd;

	dump_fd = sceIoOpen("ms0:/umem.dump", PSP_O_CREAT | PSP_O_WRONLY, 0777);

	if (dump_fd >= 0)
	{
		sceIoWrite(dump_fd, (void*) USER_START_ADDRESS, USER_END_ADDRESS - USER_START_ADDRESS + 1);
		sceIoClose(dump_fd);
	}

	
	// Open the save dialog to get more loaded libraries
	p5_open_savedata(PSP_UTILITY_SAVEDATA_SAVE);	
	
	// Changed the drive to the internal flash so that this call just fails on 
	// non-GO PSPs instead of crashing them
	dump_fd = sceIoOpen("ef0:/kmem.dump", PSP_O_CREAT | PSP_O_WRONLY, 0777);

	if (dump_fd >= 0)
	{
		sceIoWrite(dump_fd, (void*) KERNEL_START_ADDRESS, KERNEL_END_ADDRESS - KERNEL_START_ADDRESS + 1);
		sceIoClose(dump_fd);
	}
	
	// Close the save dialog
	p5_close_savedata();

	sceKernelDelayThread(100000);

	sceKernelExitGame();
}
